#include "typedef.h"

/* (c) Copyright 1989 by Jotun Hein.  These programs can be copied as long as
   this message is not deleted and it is distributed free of charge */

/* metric.c uses the metric property of the sequence space to
obtain indirect information on whether a distance is worth
calculating or not 
the uncertainty of a distance is defined as the difference between
upper and lower inequalities on the distance */

extern float getit(),corr();
float gdimaf(),upbound();
extern int known();
extern char *ecalloc(),*emalloc();
extern FILE *pout;
struct elem anchor;
int candidates;

struct metp met;

extern struct inpar parr;
extern struct param para;
extern struct globa globs;

/* 
dimaf[][] contains (i>j) upper bounds and (i<j) longest segment 
edgebound is a threshold under which all uncertainties must be
*/

metric()
{ inidimaf();
  calcmore();
}

gdima(i,j)
int i,j;
{return(met.dima[i*parr.nuseq+j]);}

pdima(i,j,v)
int i,j,v;
{met.dima[i*parr.nuseq+j]=v;}

pdimaf(i,j,v)
int i,j;
float v;
{met.dimaf[i*parr.nuseq+j]=v;}

float
gdimaf(i,j)
int i,j;
{return(met.dimaf[i*parr.nuseq+j]);}

inidimaf()
{int i,j;
 met.fracu=met.fracm=0.2;
 met.edgeboundu=(float)(para.nd*globs.lave)*met.fracu;
 met.edgeboundm=(float)(para.nd*globs.lave)*met.fracm;
for (i=0;i<parr.nuseq;i++)
 for (j=0;j<i;j++){
  pdimaf(i,j,FBIG); pdimaf(j,i,0.0);}
}

/* newedge updates the inequalities only involving the newly added sequence, 
due to the new  knowledge of (i,j). 
The time spent in metric becomes quadratic */

newedge(i,j)
int i,j;
{float m;
 struct elem *p=(&anchor),*paux;
 int pos=candidates,l;
  while (pos) 
  {l=p->next->sqno;
   m=gdimaf(i,j)+gdimaf(j,l);
   pos--;
   if (m<gdimaf(i,l)) 
    {pdimaf(i,l,m);
     pdimaf(l,i,max3(0.0,gdimaf(j,i),gdimaf(l,j))); 
    } 
   if (useful(i,l)) p=p->next;
   else {
    paux=p->next;
    p->next=p->next->next;
    candidates--;
    free(paux);
    }
 }
}


/* calcmore continues to fill up the distance matrix continuesly
calculating distance that are badly known from upper and lower
inequalities  */

calcmore()
{int i,j;
 struct elem *p;
 for (i=0;i<parr.nuseq;i++)
  {met.lowbound=FBIG; 
   listmake(i);
    while (candidates){
     p=anchor.next;
     j=p->sqno;
     pdimaf(i,j,getit(i,j,1));pdimaf(j,i,getit(i,j,1));
     if (parr.screen>1){
      printf("seq. %d versus seq. %d minimal and corrected distance: %d  %lf\n",
      i,j,gdima(j,i),gdimaf(i,j)); 
     } 
     if (gdimaf(i,j)<met.lowbound) met.lowbound=gdimaf(i,j);
     anchor.next=anchor.next->next;
     free(p);
     candidates--;
     newedge(i,j);
    }
   }
}


listmake(i)
int i;
{int j;
struct elem *p;
for (j=0;j<i;j++){
 p=(struct elem *)emalloc(sizeof(struct elem));
 p->sqno=j;
 p->next=anchor.next;
 anchor.next=p;
 }
candidates=i;
}


/* uncertainty calculates the difference betwee the upper and lower
inequality on distance i,j */

useful(i,j)
int i,j;
{float runc,rmin,rmax;
 
if (i>j) {runc=2.0*(gdimaf(i,j)-gdimaf(j,i));
	  rmax=gdimaf(i,j);
          rmin=rmax-runc;
         }
else     {runc=2.0*(gdimaf(j,i)-gdimaf(i,j));
	  rmax=gdimaf(j,i);
          rmin=rmax-runc;
	 }  
     
     return(rmin<met.lowbound && rmin<met.edgeboundm);
}


/* findmin finds the pair of sequences that are closest according to the known 
   distances */

findmin()
{int i,j;
 int small=SBIG;
 for (i=0;i<parr.nuseq;i++)
  for (j=i+1;j<parr.nuseq;j++)
   if (known(i,j)) 
    if (known(i,j)<small){
                          met.newseq=i;
                          met.closeseq=j;
                          small=known(i,j);
                         }
}

float upbound(i,j)
int i,j;
{if (i>j) return(gdimaf(i,j));
 else     return(gdimaf(j,i));
}


/* createonelist creates a list, with all sequences except the closest pair 
   As the tree grows the list shrinks containing only sequences not in the tree */ 

createonelist()
{int i;
 struct outtree *p;
 met.otreel=parr.nuseq-2;
 for (i=0;i<parr.nuseq;i++)
  if (i!=met.newseq && i!=met.closeseq){
                      p=(struct outtree *)emalloc(sizeof(struct outtree));
                      p->sqno=i;
                      p->sqclose=met.closeseq;
                      p->dist=upbound(met.closeseq,i); 
                      p->next=met.otree.next;
                      met.otree.next=p;
                     }
}

/* nextpair finds the pair of sequences one in the tree, one not, that are 
   closest */

nextpair()
{int j;
 float small=FBIG;
 struct outtree **po,**p,*pa;
  for (j=1,po=(&met.otree.next);j<=met.otreel;j++,po=(&(*po)->next))
    if ((*po)->dist<small){
                          met.closeseq=(*po)->sqclose;
                          met.newseq=(*po)->sqno;
                          small=(*po)->dist;
                          p=po;
                          }
pa=(*p);
(*p)=(*p)->next;
free(pa);
met.otreel--;
growpair();
}

/* growpair removes one sequence from otree and updates it */

growpair()
{int j;
 struct outtree *p;
  for (j=1,p=met.otree.next;j<=met.otreel;j++,p=p->next)
   {p->dist=min(upbound(met.newseq,p->sqno),p->dist);
    if (p->dist==upbound(met.newseq,p->sqno)) p->sqclose=met.newseq;
   }
}

