# include "typedef.h"

/* (c) Copyright 1990 by Jotun Hein.  This program can be copied as long as
   this message is not deleted and it is distributed free of charge */

/* stati.c reads form result.dat, chooses one history from the set of possible
   histories, constructs one of the possible alignments corresponding to this
   history, calculates certain statistics on the history, writes them out
   together with the alignment on alignment.dat. It also writes out the 
   phylogeny on treefile.dat in a form that can be used by 
   Felsenstein's graphics programs */

extern int known(),di();
extern FILE *pphyl,*pout,*pin,*par;
extern char logtide(),*emalloc(),cv();
extern char *alig;
extern float finv();

extern struct outstat oust;
extern struct globa globs;
extern struct inpar parr;
extern struct treep t;

/* next four procedures updates list of numbers of indels - 
   it can be done simpler  */

locate(i)
int i;
{oust.indelpos=0;
 oust.indelpoint=(&oust.indels);
 while (oust.indelpos!=oust.indell && oust.indelpoint->next->pos<i)
  {oust.indelpos++; oust.indelpoint=oust.indelpoint->next;}
}

insertlist(i)
int i;
{struct integlist *p;
 if (nextmatch(i))
  {p=(struct integlist *)emalloc(sizeof(struct integlist));
   p->next=oust.indelpoint->next;
   p->pos=i;
   p->val=1;
   oust.indell++;
   oust.indelpoint->next=p;
  }
 else oust.indelpoint->next->val++;
}

int nextmatch(i)
int i;
{int r=0;
 if (oust.indelpos==oust.indell) r=1;
 else if (oust.indelpoint->next->pos!=i) r=1;
 return(r);
}

anotherindel(i)
int i;
{locate(i);
 insertlist(i);
}

/* calculates statistics of interest on the chosen history */

statistics()
{int i;
 if (parr.udrest) {
 if (parr.nuseq<MATSIZE) outmat();
 fprintf(pout,"weight of insertion-deletion of length k: %d + k*%d\n",
               parr.a,parr.b);   
 fprintf(pout,"number of sequences: %d   \n",parr.nuseq);   
 fprintf(pout,"total number of pairwise comparisons used: %d   \n",t.comptot);   
 fprintf(pout,"%d is the total length of alignment\n",globs.aligtot);
 }
 for (i=0;i<=globs.aligtot;i++){oust.poiar[i][0]=oust.poiar[i][1]=0;}
 eventcount(t.top);
 poiappr();
 t.top->up=0;
 if (parr.nuseq>2) tegntrae();
 if (parr.udmat){
 table1();
 printlist();   
 fprintf(pout,"total weight of history: %d   \n\n\n",globs.totdist);   
 fprintf(pout,"ALIGNMENT OF SEQUENCES: \n\n");   
 }
} 

/* calculates the fraction of matching sites relative to comparable
   sites between p1 and p2 */

percent(p1,p2)
int p1,p2;
{int totmatch=0,totscore=0,i;
 for (i=1;i<=globs.aligtot;i++)
  if (alig[p1*globs.nmax+i]!='-' && alig[p2*globs.nmax+i]!='-') 
   {totmatch++;
    if (alig[p1*globs.nmax+i]==alig[p2*globs.nmax+i]) totscore++;
   }
  return((100*totscore)/totmatch);
}

/* eventcount counts the events on branches p->left and p->right */

eventcount(p)
struct node *p;
{if (!p->tip){
  pairpare(p->sqno,p->left->sqno,p->left);
  pairpare(p->sqno,p->right->sqno,p->right);
  eventcount(p->left);
  eventcount(p->right);
 }
}

/* delcount counts one indel on branchs p->up */

delcount(l,p)
int l;
struct node *p;
{anotherindel(l);
 p->up+=((int)parr.a+l*(int)parr.b);
}

/* pairpare compares sequence s1 with sequence s2, which are
connected by branch p->up */

pairpare(s1,s2,p)
int s1,s2;
struct node *p;
{int t,l,i=0;
p->up=0;
while (i<=globs.aligtot){l=0;
 while (alig[s1*globs.nmax+i]==alig[s2*globs.nmax+i] && i<=globs.aligtot) i++;
 if (i<=globs.aligtot){t=classify(alig[s1*globs.nmax+i],alig[s2*globs.nmax+i]);
  if (t==0) {substcount(alig[s1*globs.nmax+i],alig[s2*globs.nmax+i],p);i++;}
  if (t==1)
     {l=0; while (alig[s2*globs.nmax+i]=='-' && i<=globs.aligtot){i++;
      if (alig[s1*globs.nmax+i]!='-') l++;}
      delcount(l,p);           
      }     
  if (t==2)
     {l=0; while (alig[s1*globs.nmax+i]=='-' && i<=globs.aligtot){i++;
      if (alig[s2*globs.nmax+i]!='-') l++;}
      delcount(l,p);           
      }     
  }    
 }

for (i=0;i<=globs.aligtot;i++)
 if (alig[s1*globs.nmax+i]!='-' || alig[s2*globs.nmax+i]!='-'){
  if (alig[s1*globs.nmax+i]=='-' || alig[s2*globs.nmax+i]=='-'){
   oust.poiar[i][0]+=parr.b+2;
   oust.poiar[i][1]+=p->up;
  }
  else{
   oust.poiar[i][0]+=di(amino(alig[s1*globs.nmax+i]),amino(alig[s2*globs.nmax+i]));
   oust.poiar[i][1]+=p->up;
  }
 }
}

/* poi(sson)appr(oximation) determines how many standard deviations from the 
   mean amount of evolution expected at a site the observed amount of evolution
   is given that all sites has the same mutability and the mutation proces is a
   poisson process  */
 
poiappr()
{int i;
 float large,small,e,s;
 int r;
 for (i=0;i<=globs.aligtot;i++){
  large=(float)oust.poiar[i][1]/2.0;
  small=(float)oust.poiar[i][0]/2.0;
  e=large/(float)globs.lave;
  s=sqrt(e);
  r=4-(int)((small-e)*finv(0.5*s));
  oust.poiar[i][2]=r;
 } 
}

/* checks if a position in the alignment is of type 0,1 or 2 */

classify(n1,n2)
char n1,n2;
{ int t;
 if (n1=='-') t=2;
 if (n2=='-') t=1;
 if (n1!='-' && n2!='-') t=0;
 return(t);
}

/* counts weight of substitutions occuring between seq i and j */

substcount(i,j,p)
char i,j;
struct node *p;
{if (i!=j){p->up+=(int)di(amino(i),amino(j));
 oust.substi[logtide(amino(i))][logtide(amino(j))]++;}
}

