# include "typedef.h"

/* (c) Copyright 1990 by Jotun Hein.  This program can be copied as long as
   this message is not deleted and it is distributed free of charge */

/* story.c chooses one history from the set of possible
   histories, constructs one of the possible alignments corresponding to this
   history, calculates certain statistics on the history, writes them out
   together with the alignment on alignment.dat. It also writes out the 
   phylogeny on treefile.dat in a form that can be used by 
   Felsenstein's graphics programs */

extern int known(),di();
extern FILE *pphyl,*pout,*pin,*par;
extern char logtide(),*emalloc(),cv();

struct nuclink *findconnect();
char *alig;

extern struct treep t;
extern struct outstat oust;
extern struct aliglospe ags;
extern struct alilocspe als;
extern struct sekvens *sek;
extern struct param para;
extern struct inpar parr;
extern struct globa globs;

/* governs choice of one history and the generation of output */

inout()
{ags.s3=t.top->sqno;
 als.sg=sek[ags.s3].s;
 alicon();
 statistics();
 outal();
 fprintf(pout,"number of completely conserved sites: %d\n",oust.conssite);
}

/* alicon governs the choice of one history in the set of equally 
   parsimoneous histories calculated */

alicon()
{int i,j;
 alfill();
 for (i=0;i<=para.signnu;i++)
  for (j=0;j<=para.signnu;j++)
   oust.substi[i][j]=0;
 topseq(sek[t.top->sqno].p->nl,t.top);
 for (i=0;i<parr.alseq;i++) sek[i].len=sek[i].p->nl=0;
}

/* chooses a path in the top seq.gr. and calls addone for each real edge */

topseq(l,p)
int l;
struct node *p;
{if (l){topseq(als.sg[l].first->nextnode,p);
        if (als.sg[l].first->type>=0) addone(p,als.sg[l].first);
       }
}

/* goes down in the subtree hanging from each edge */

avalance(pr,pn)
struct node *pr;
struct nuclink *pn;
{if (!pr->tip){
  if (pn->nextnode) down(pr,pn);
  if (pn->type==0){avalance(pr->left,pn->left);
                   avalance(pr->right,pn->right);
                  }
  if (pn->type==1) avalance(pr->left,pn->left);
  if (pn->type==2) avalance(pr->right,pn->right);
 }
 pr->last=pn;
}

/* adds one position to the alignment */

addone(pr,p)
struct nuclink *p;
struct node *pr;
{
 avalance(pr,p);
 p->nucset=first(p->nucset);
 fire(pr,p);
 globs.aligtot++;
 if (globs.aligtot==globs.nmax) printf("alignment too big. Increase nmax!\n");
}

/* checks if an edge is contigous with the last added edge, if not finds
   the missing bridge */

down(pr,pn)
struct nuclink *pn;
struct node *pr;
{
 if (pn->type==1 || !pn->type)
    if (!contig(pr->left,pn->left))
       bfs(pr,pn->nextnode,ancnode(pr),1);

 if (pn->type==2 || !pn->type)
    if (!contig(pr->right,pn->right))
       bfs(pr,pn->nextnode,ancnode(pr),2);
}

/* given a sequence, seq, and an edge p, the node owning the edge is found */

int
ancnode(pr)
struct node *pr;
{struct nuclink *poi,*p=pr->last;
 int i,r,found=0;
 struct nucnode *sga=sek[pr->sqno].s;

 if (p==NULL) 
    {i=1;
       while (!found){
          poi=sga[i].first;
          while (poi!=NULL){
             if (!poi->nextnode) {found=1;r=i;}
             poi=poi->next;
            }
          i++;
         }
      i=0;
      }

 else {i=p->nextnode+1;
       while (!found){
          poi=sga[i].first;
          while (poi!=NULL){
             if (poi==p) {found=1;r=i;}
             poi=poi->next;
            }
          i++;
         }
      }
 return(r);
}

/*  given an edge contig determines if it just precedes a node that has been
    processed otherwise an indel must be postulated */

int contig(pr,pn)
struct nuclink *pn;
struct node *pr;
{int j=0;
 struct nuclink *p;
 struct nucnode *sga=sek[pr->sqno].s;
 p=sga[pn->nextnode].first; 
 while (p!=NULL){
    if (p->type>=0)  j=((p==pr->last) || j); 
    else j= (j || contig(pr,p));
    p=p->next;
   }
 
 return(j);
}

/* following procedures performes a breath-first-search to find the shortest
   indel path needed. Matpos is used again, with two enumeration systems: 
   actual node number and number as encountered by bfs. posi[n1][0] is 
   visited/not visited, posi[n2][3]=n1, posi[x][2] is ancestor to x,
   posi[][1] is the generation number of the node
   pos1 is lower index of nodes and pos2 is higher index of nodes */

/* the connecting bridge between x and y is found */

struct nuclink *
findconnect(x,y,seq)
int x,y,seq;
{struct nucnode *sga=sek[seq].s;
 struct nuclink *p,*pr=NULL;
 p=sga[y].first;
 while (p!=NULL){
    if (p->nextnode==x && p->type>=0) pr=p;
    p=p->next;
   }
 return(pr);
}

/* bfs bridges b and e using only edges of type t or dummy edges */

bfs(pr,b,e,t)
struct node *pr;
int b,e,t;
{int x,y,i,pos1=0,pos2=0,(*posi)[4];
 struct nuclink *poi,*p;
 struct nucnode *sga=sek[pr->sqno].s;
 posi=(int (*) [4])emalloc((b-e+1)*4*sizeof(int));
for (i=b-e;i>=0;i--) {posi[i][0]=0; posi[i][1]=SBIG;}
 posi[b-e][1]=0;
 posi[0][3]=b;
 while (pos1!=(pos2+1)){ /* this condition would emply array is empty */
  x=posi[pos1][3];
  pos1++; /* this is deleting an element from the queue */
  poi=sga[x].first;
  while (poi!=NULL)
     {if (!posi[poi->nextnode-e][0] && poi->type==t && poi->nextnode>=e){
         posi[poi->nextnode-e][0]=1;
         pos2++;posi[pos2][3]=poi->nextnode; /* a node is added to the queue */
         if (posi[poi->nextnode-e][1]>(posi[x-e][1]+1))
	    {posi[poi->nextnode-e][1]=posi[x-e][1]+1;
	     posi[poi->nextnode-e][2]=x;
            }
         }
      poi=poi->next;
     }
 }

x=e;
while (x<b){
  y=posi[x-e][2];
  p=findconnect(x,y,pr->sqno);
  if (p==NULL) 
     {printf("UPS ebxy %d %d %d %d\n",e,b,x,y); x=b;}
  else {
     if (p->type==1 && t==1) addone(pr->left,p->left);
     if (p->type==2 && t==2) addone(pr->right,p->right);
     x=y;
   }
 }
free(posi);
}

/* preceeding procedures performs a breath-first-search */

/* given an edge with a subtree hanging from it, an assignment of single 
   nucleotides is found to all nodes in the subtree */

fire(pt,pn)
struct node *pt;
struct nuclink *pn;
{int pos;
 alig[pt->sqno*globs.nmax+globs.aligtot]=cv(pn->nucset);
 pos=logtide(pn->nucset);
 if (!pt->tip){
  if (pn->type==0){
   triple(&pn->left->nucset,&pn->right->nucset,pos);
   fire(pt->left,pn->left);
   fire(pt->right,pn->right); 
  }
  if (pn->type==1) {dobbelt(&pn->left->nucset,pos);fire(pt->left,pn->left);}
  if (pn->type==2) {dobbelt(&pn->right->nucset,pos);fire(pt->right,pn->right);}
 }
}

