#include "typedef.h"

/* (c) Copyright 1989 by Jotun Hein.  These programs can be copied as long as
   this message is not deleted and it is distributed free of charge */

/* semitria.c contructs phylogenies and directs which sequences should be 
   aligned */

extern float getit(),finv();
extern FILE *pout,*pin,*pphyl;
int known();
float priva(),atpos();
extern char *emalloc();
int heapsize,tipno;
struct auxheap *heap;

struct treep t;

extern struct param para;
extern struct metp met;
extern struct inpar parr;
extern struct globa globs;

/* maketree construct a distance tree by adding sequences to a growing tree */

fasttree()
{t.comptree=t.comptot;
 threetree();
 for (t.treeno=3;t.treeno<parr.nuseq;t.treeno++){
  startpoint();
  t.mindis=getit(met.newseq,met.closeseq,1);
  if (parr.screen) 
   printf("positioning the %d'th seq.: %d which is %.1lf from %d"
   ,t.treeno,met.newseq,t.mindis,met.closeseq); 
  newbranch(); 
  if (parr.screen) printf(" using %d comp.\n",t.comptot-t.compr); 
 t.compr=t.comptot;
 }
}


/* creates distance tree for the first three sequences */

threetree()
{float r;
 findmin();
 createonelist();
 t.plt.sqno=met.newseq;
 t.plt.left=(struct auxnode *)emalloc(sizeof(struct auxnode));
 t.plt.left->left=(struct auxnode *)emalloc(sizeof(struct auxnode));
 t.plt.left->right=(struct auxnode *)emalloc(sizeof(struct auxnode));
 t.plt.left->up=(&t.plt);
 t.plt.left->left->sqno=met.closeseq;
 t.plt.left->sqno=(0-2-parr.nuseq);
 t.legs[0]=t.plt.leftlen=t.plt.left->uplen=
      getit(met.newseq,met.closeseq,1);
 nextpair();
 t.x[0]=getit(t.plt.sqno,met.newseq,1);
 t.x[1]=getit(met.closeseq,met.newseq,1);
 t.plt.left->leftlen=t.plt.left->left->uplen=atpos(0,1);
 t.plt.leftlen=t.plt.left->uplen=t.plt.leftlen-atpos(0,1);
 t.plt.left->right->sqno=met.newseq;
 t.plt.tip=t.plt.left->left->tip=t.plt.left->right->tip=1;
 t.plt.left->tip=0; 
 t.plt.left->left->up=t.plt.left;
 t.plt.left->right->up=t.plt.left;
 r=priva(met.newseq,t.plt.left);
 t.plt.left->rightlen=t.plt.left->right->uplen=r;
}

startpoint()
{nextpair();
 if (met.closeseq==t.plt.sqno){t.lnode=(&t.plt);
                               t.nnode=t.plt.left;}
 else {findit(t.plt.left); 
       t.nnode=t.lnode->up;}
}

/* finds tip with closeseq in it and makes lastnode point to it */

findit(p)
struct auxnode *p;
{if (!p->tip){ 
  findit(p->left);
  findit(p->right);
 }
 else if (p->sqno==met.closeseq) t.lnode=p;
}

/* newbranch adds sequence number n to subtree Tn_1 */

newbranch()
{prepare();
 decide();
}

/* finds closest tip to newseq, weighted distance to newseq and 
   weighted depth of subtree */


/* bladlist */

dtree(i,pos)
int i,pos;
{float dp;
 struct auxnode *p;
 int j,dad;
 dp=heap[pos].depth;
 p=heap[pos].p;
 dad=heap[pos].daddy;
 if (!p->tip){
  vaek(pos); 
  if (p->up->sqno!=dad)
   paaplads(p->sqno,p->up,dp+p->uplen);
  if (p->left->sqno!=dad)
   paaplads(p->sqno,p->left,dp+p->leftlen);
  if (p->right->sqno!=dad)
   paaplads(p->sqno,p->right,dp+p->rightlen);
 }
 j=faerdig(); 
 if (j) dtree(i,j);
} 


vaek(pos)
int pos;
{int i;
 for (i=pos;i<=heapsize;i++) heap[i]=heap[i+1];
heapsize--;
}


paaplads(sq,p,dp)
int sq;
struct auxnode *p;
float dp;
{int i=heapsize+1;
 while (heap[i-1].depth<dp){
  heap[i]=heap[i-1];
  i--;
 }
 heap[i].p=p;
 heap[i].depth=dp;
 heap[i].daddy=sq;
 heapsize++;
}


int faerdig()
{int j=heapsize;
 while (j && heap[j].p->tip) j--;
 tipno=heapsize-j;
 if ((heap[j+1].depth>t.mindis && tipno) || !j) j=0;
 return(j);
}


depthtree(o,se,dp,p)
int o,se;
float dp;
struct auxnode *p;
{struct auxnode *paux;
 int j;
 float y,d,w,wtot=0.0;
 heap[0].depth=FBIG; 
 heap[1].p=p;
 heap[1].daddy=o;
 heap[1].depth=dp;
 heapsize=1;
 t.arms[se]=t.x[se]=0.0;
 dtree(se,1);
 for (j=1;j<=tipno;j++){paux=heap[heapsize-j+1].p;
		       y=getit(paux->sqno,met.newseq,1);
		       d=heap[heapsize-j+1].depth;
		       w=finv(y);
		       w*=w;
		       wtot+=w;
		       t.x[se]+=y*w;
	               t.arms[se]+=d*w;}
  t.arms[se]/=wtot;
  t.x[se]/=wtot;
}


/* attpoint puts in an additional branch in the tree with seq. n at the
   tip when it's position etc. has been determined. Let U be the upper
   node and L be the lower node. Then attpoint introduces an additional
   node (*pha) between these, that above it has U and right down has L
   and left down has (*pht) the tip with the new node. All pointers and 
   lengths are updated. The length between (*pha) and (*pht) will be
   privd, the distance between (*pha) and U is res or resop, the length
   beween L and (*pha) is resop or res, dependent on the orientation
   of the segment */ 

attpoint(d)
int d;
{struct auxnode *pha,*pht,**pd,**pu;
 float  *dd,*du,upl,downl,pri;
 if (t.vlen[d]<=0.0) t.vlen[d]=0.0;
 if (d==0){
  if (t.vlen[0]>t.nnode->uplen) t.vlen[0]=t.nnode->uplen; 
  if (t.nnode->up->left==t.nnode) 
    {dd=(&t.nnode->up->leftlen);pd=(&t.nnode->up->left);}
  else {dd=(&t.nnode->up->rightlen);pd=(&t.nnode->up->right);}
  upl=t.vlen[0];
  downl=t.nnode->uplen-upl;
  pu=(&t.nnode->up);
  du=(&t.nnode->uplen);
 }
 if (d==1){
  if (t.vlen[1]>t.nnode->leftlen) t.vlen[1]=t.nnode->leftlen; 
  pd=(&t.nnode->left);
  pu=(&t.nnode->left->up);
  downl=t.vlen[1];
  upl=t.nnode->leftlen-downl;
  dd=(&t.nnode->leftlen);
  du=(&t.nnode->left->uplen);
 }
 if (d==2){
  pd=(&t.nnode->right);
  pu=(&t.nnode->right->up);
  downl=t.vlen[2];
  upl=t.nnode->rightlen-downl;
  dd=(&t.nnode->rightlen);
  du=(&t.nnode->right->uplen);
 }

 pha=(struct auxnode *)emalloc(sizeof(struct auxnode));
 pht=(struct auxnode *)emalloc(sizeof(struct auxnode));
 pht->up=pha;
 pht->tip=1;
 pht->sqno=met.newseq;
 pha->right=(*pd);
 pha->rightlen=pha->right->uplen=upl;
 pha->up=(*pu);
 pha->uplen=downl;
 pha->tip=0;
 pha->sqno=3-t.treeno-parr.nuseq;
 pha->left=pht;
 *dd=downl;
 *du=upl;
 (*pd)=pha;
 (*pu)=pha;
 pri=priva(met.newseq,pha);
 pht->uplen=pha->leftlen=pri;
}


prepare()
{float r1,r2,w1,w2;
t.legs[0]=t.nnode->uplen;
t.legs[1]=t.nnode->leftlen;
t.legs[2]=t.nnode->rightlen;

depthtree(t.nnode->sqno,0,0.0,t.nnode->up);
depthtree(t.nnode->sqno,1,0.0,t.nnode->left);
depthtree(t.nnode->sqno,2,0.0,t.nnode->right);

w1=finv(t.x[0]+t.x[1]);w1*=w1;
w2=finv(t.x[0]+t.x[2]);w2*=w2;
t.vlen[0]=(atpos(0,1)*w1+atpos(0,2)*w2)/(w1+w2);

w1=finv(t.x[1]+t.x[0]);w1*=w1;
w2=finv(t.x[1]+t.x[2]);w2*=w2;
r1=atpos(1,0);
r2=atpos(1,2);
t.vlen[1]=(r1*w1+r2*w2)/(w1+w2);

w1=finv(t.x[2]+t.x[0]);w1*=w1;
w2=finv(t.x[2]+t.x[1]);w2*=w2;
t.vlen[2]=(atpos(2,0)*w1+atpos(2,1)*w2)/(w1+w2);

}


float
atpos(a,b) 
int a,b;
{float add,ult,y,r,z; 
 y=t.arms[a]+t.arms[b]+t.legs[a]+t.legs[b];
 z=max3(y,t.x[a],t.x[b]);
 add=(y+t.x[b]-t.x[a])/2;
 if (t.x[a]<=t.x[b] && t.x[a]<=y) ult=y-t.x[a]/2.0;
 if (t.x[b]<=t.x[a] && t.x[b]<=y) ult=t.x[b]/2.0;
 if (y<=t.x[a] && y<=t.x[b]) ult=y/2.0;
 if (z<=0.4*para.nd*globs.lave) r=add;
 else if (z<=2.2*para.nd*globs.lave) 
  r=add + ((ult-add)*(z-0.4*para.nd*globs.lave)/(1.8*para.nd*globs.lave));
 else r=ult;
 if (r<0.0) r=0.0;
 if (r>y) r=y;
 return(r-t.legs[b]-t.arms[b]);
 }


float
priva(o,p) 
int o;
struct auxnode *p;
{float r;
 depthtree(o,3,0.0,p);
 r=t.x[3]-t.arms[3];
 return(r);
 }

/* decide determines if the attachmentpoint of the new sequence has been
   found or the search should go on down in some subtree  */

decide()
{int i,v;
 float largest=(-1000000.0);
 struct auxnode *memnode;
 for (i=0;i<=2;i++)
   if (largest<t.vlen[i]){
    largest=t.vlen[i];
    v=i;
   }
         memnode=t.nnode;
	 if (v==0) {t.mindis-=t.nnode->uplen;t.nnode=t.nnode->up;}
         if (v==1) {t.mindis-=t.nnode->leftlen;t.nnode=t.nnode->left;}
         if (v==2) {t.mindis-=t.nnode->rightlen;t.nnode=t.nnode->right;}
         if (t.lnode==t.nnode || t.nnode->tip) 
	    {t.nnode=memnode;attpoint(v);}
	 else {
	  t.lnode=memnode;
	  prepare();
          decide();
          }
}

