.TH treealign  
\ "Version of Nov 23  1990" \" 
.\" man page written by Peter A. Cejchan
.\"

.SH NAME
treealign: does Parsimony Tree Reconstruction and Pairwise Alignment simultaneously.

.SH SYNOPSIS

 
\fBtreealign\fP
.LP


.SH DESCRIPTION
The program performs phylogenetic
alignment of homologous sequences (DNA/Protein) as described in the articles
"A Unified Approach to Alignments and Phylogenies" in Methods of Enzymology,
Spring 90 (VOL 183) and two articles in Mol.Biol.Evol(Nov.89). The first 
article is the better starting point.It solves following two problems simultaneously:

Parsimony Tree Reconstruction:  A very predominant method of choosing a 
phylogeny for a set of sequences is to find the phylogeny that postulates the
least change.  Such programs have been made since the late 60s/early 70s, but 
they could only be applied to prealigned sequences or sequences, that for some
reason had not been subject to insertion-deletions.  

Pairwise alignment: Since late 60s/early 70s (again) methods have existed that
could find the set of substitutions and insertion-deletions that represented the
least evolution or the two sequences.  This is equivalent to align two 
sequences.  Unfortunately this was restricted to two sequences.

It can analyze both DNA and protein (either or, though).  I have run all 
existing globins (>300) and up to 20 complete retroviuses (8-10kb (DNA). The 
last analysis took about 2hrs CPU time, and in this case it is regrettable, that
it cannot analyze sequences that is a mix of DNA and protein. But it should be
able to handle most reasonable alignment problems. 
.IP

.SH WHAT IT DOESN'T
There are things the program cannot do or should not be used for:

i: It cannot test whether homologies are statistically significant,
it can only reconstruct phylogenies and make alignments.  The sequences should
be homologous and there should be a history to be found. If you give it a set
of COMPLETELY unrelated sequences, it is possible that it will not be able to
align them, since it cannot allocate enough memory.   

ii: The sequences should not vary in length because they have been sequenced
unequally much.  Length differences should be due to evolution. Thus it should
not be used to look for local homologies.

iii: Sequences that has been subject to recombinations or gene conversions
should be treated with caution.

iv: It is part of the the principle of reconstructing the history of a set
of sequences that nucleotides are only put in the same column if the are
same column are connected by a path in the tree where the nucleotide has
neither been inserted or deleted. This has consequences, that occasionnally
gives an alignment, that people feel could be improved, i.e. could be changed
a little to look 'nicer'. To illustrate the phenomena, assume 100 sequences
is to be aligned.  98 have length 0 and 2 have length 1 and consists of a 'T'.
How should these 100 sequences be aligned.  If the 2 'T's are brothers in the
tree, they will be in the same column, their immediate ancestor is a 'T' and
all other positions in the column are occupied by gap signs.  The weight of
the alignment corresponds to one insertion-deletion.  On the other hand if the
2 'T's are at tips very distant in the tree, they should be in two different
columns, and the weight of the alignment will correspond to 2 insertion-
deletions. Had they been put in the same column all internal nodes in the
path connecting them will be assigned 'T's and a large set of insertion-
deletions would be postulated, since the 'T's only survived in two tips.
.LP

.SH FILES
Three files must be present to run the program:
i) par.dat.
ii) file with sequences.
iii) mut.dat file with mutation distance matrix. Mut.dat is not shown as there
     is no choice presently for the user to be made concerning this file.

a fourth file must be present if the option usertree is to be used:
iv) file with user specified tree.

Decide with which parameters the program should be run and write these
parameters in the file par.dat.  The program is completely non-interactive.
Make the file with the input sequences and be careful that they have the right
format.
.LP
.SH EXAMPLE
Next line is the first line in an example of par.dat, the first input file:
1  4   8   3     
0  0  
test1.pro
test.tree
test.ali
user.tree

PRO(1)/DNA(0) nuseq a b  
ancestorout usertree 
fileseq
filetree
fileali
usertreefile


COMMENTS ON PARAMETER VALUES IN THE par.dat EXAMPLE

protein or DNA corresponds to 1 and 0, respectively. Thus protein in this case

nuseq is number of sequences to be found on fileseq. 4 in this case.

a,b: gap penalty function is gk=a+k*b.  8 + 3*k in this case. b should not
     be put to less than 3. a should just be non-negative.

ancesterout: only present sequences in alignment (0) or also ancestral
             sequences (1).

usertree: should the program reconstruct the phylogeny (0) or will it be
          supplied (1). No in this case, phylogeny must be reconstructed.
          It is possible to use this option and then try to find a better
          tree by setting parcyc larger than 0.

fileseq: file with sequences on it. test1.pro in this case.

filetree: file where  phylogeny will be written. test.tree in this case.

fileali: file where main output will be written. test.ali in this case.

user.tree: name of file with usertree. user.tree in this case, but not needed.
.LP
Next line is first line on test1.pro, the second inputfile
>
HEMOGLOBIN ALPHA CHAIN - HUMAN AND CHIMPANZEES
 V L S P A D K T N V K A A W G K V G A H A G E Y G A E A L E
 R M F L S F P T T K T Y F P H F D L S H G S A Q V K G H G K
 K V A D A L T N A V A H V D D M P N A L S A L S D L H A H K
 L R V D P V N F K L L S H C L L V T L A A H L P A E F T P A
 V H A S L D K F L A S V S T V L T S K Y R *

>
HEMOGLOBIN BETA CHAIN - HUMAN, CHIMPANZEES, AND GORILLA
 V H L T P E E K S A V T A L W G K V N V D E V G G E A L G R
 L L V V Y P W T Q R F F E S F G D L S T P D A V M G N P K V
 K A H G K K V L G A F S D G L A H L D N L K G T F A T L S E
 L H C D K L H V D P E N F R L L G N V L V C V L A H H F G K
 E F T P P V Q A A Y Q K V V A G V A N A L A H K Y H *

>
MYOGLOBIN - HUMAN
 G L S D G E W Q L V L N V W G K V E A D I P G H G Q E V L I
 R L F K G H P E T L E K F D K F K H L K S E D E M K A S E D
 L K K H G A T V L T A L G G I L K K K G H H E A E I K P L A
 Q S H A T K H K I P V K Y L E F I S E C I I Q V L Q S K H P
 G D F G A D A Q G A M N K A L E L F R K D M A S N Y K E L G
 F Q G *

>
LEGHEMOGLOBIN I - YELLOW LUPIN
 G V L T D V Q V A L V K S S F E E F N A N I P K N T H R F F
 T L V L E I A P G A K D L F S F L K G S S E V P Q N N P D L
 Q A H A G K V F K L T Y E A A I Q L E V N G A V A S D A T L
 K S L G S V H V S K G V V D A H F P V V K E A I L K T I K E
 V V G D K W S E E L N T A W T I A Y D E L A I I I K K E M K
 D A A *
The line above is the last line read on test1.pro, as it contains end of
fourth sequence.

A sequence must obey following format.: A ">" on FIRST position on a line,
the sequence starts on second line after that and continues until a "*" is
encountered. Always check if the sequences obey the format and there are enough
sequences on the specified file.

Next line is the only line in user.tree, the fourth and optional inputfile.
((1,2)6,(3,4)5)7;

This describes the topology of the phylogeny in test.tree.


.SH LIMITS 
There are a few internal limits in the program:

  If more then 40 sequences are analyzed, the program will not draw the tree. 
This is determined by the constant PHYSIZE in typedef.h

  If more than 22 sequences are analyzed, the program will not write out a
distance matrix. This is determined by the constant MATSIZE in typedef.h

.SH COPYRIGHT
(c) Copyright Nov. 1990 by Jotun Hein.  These programs can be copied as long as
   this message is not deleted and the programs are distributed free of charge.


.SH AUTHOR

.nf
Jotun Hein,National Institue of Genetics, Mishima, Shizuoka-Ken, 411 Japan
JHEIN@NIGUTS.NIG.AC.JP
.fi





