# include <stdio.h>
# include <math.h>

# define SBIG 100000
# define FBIG 100000.0
# define SHLEN 25 /* maximum number of preds. or desc. in segment graph */ 
# define MXPUFF 50 + (globs.nmax/50)
# define MEMORY 1000000
# define STLG 1000 /* this is the maximum length of inpu-tree-string */
# define MAXSIGNS 25
# define MATSIZE 22 /* distance matrix for more sequence will not be drawn */
# define PHYSIZE 40 /* trees for more sequences will not be drawn */ 


# define max(A,B) ((A) > (B) ? (A) : (B))
# define min(A,B) ((A) < (B) ? (A) : (B))

# define min3(A,B,C) ((A) < (B) ? min(A,C) : min(B,C))
# define max3(A,B,C) ((A) > (B) ? max(A,C) : max(B,C))

# define min4(A,B,C,D) (min(min(A,B),min(C,D)))

# define coor(x,y) ((x<0 || y<0 || x<dx[y] || x>dy[y]) ? 0 : (row[y]+x))

struct node{
 int wei;
 int sqno,tip,up,done,edge;
 struct node *left,*right;
 float dislen;
 struct nuclink *last;
};

typedef struct nuclink{
 int nextnode,nesc,type;
 int nucset;
 struct nuclink *next,*left,*right;
}*arrow;


struct nucnode{
 struct nuclink *first;
};

struct auxnode {
 float uplen,leftlen,rightlen;
 struct auxnode *up,*left,*right;
 int sqno,tip;
};

struct elem{
   int sqno;
   struct elem *next;
   };

 struct integlist{
  int pos,val;
  struct integlist *next;
 };

 struct outtree{
  int sqno,sqclose;
  float dist;
  struct outtree *next;
 };

 struct alinode{
 int w;
 int start[3];
 struct elem *next;
};

struct hali{
 int kind,num;
 struct hali *next;
 struct elem *first;
};

struct anch{
 int num;
 struct hali *first;
};

struct auxheap {
 struct auxnode *p;
 float depth;
 int daddy;
 };

/* metp: all variables special to metric */

struct metp{
   float *dimaf,lowbound,edgeboundm,edgeboundu,fracu,fracm;
   int closeseq,*dima,newseq,otreel;
   struct outtree otree;
  };

/* treep:  all variables used in tree construction 

parcycmax   : max. no of cycles in improving parsimony tree
discycmax   : max. no of cycles in improving distance tree
comptree    : no. pairwise comparisons so far
compr       : last value of comptree printed to screen
treeno      : size of present distance tree
top         : root of parsimony tree
prelimtree  : root of distance tree
*/

struct treep{
   int cyclno,comptot,parcycmax,discycmax,comptree,compr,treeno;
   struct node *top;
   struct auxnode plt,*nnode,*lnode;
   float inhibit,mindis,vlen[3],arms[4],x[4],legs[4]; 
  };

/* align global special */

struct aliglospe{
  int s[2],s3,w;
  int l[2],stln[2];
  char *(se [2]);
  struct nucnode *sg[2],*sg3;
 };

/* align local special */

struct alilocspe{
 int b1,b2,l1,l2,stln1,stln2;
 struct nucnode *sg,*sg3;
};

/* domain: all inf. nesc. to characterize a domain

num     : number of domain in sequence (gene)
sety    : type of sequence (coding/noncoding)
al,nl   : lenght of amino/nucleotide sequence
se,tr   : start of domain in very long sequence (sequence and translated seq)
seg,segtr pointers to sequence graphs
*/

struct domain{
  int num,sety,al,nl;
  int se,tr;
  struct node *seg,*segtr;
  struct domain *next;
 };


struct sekvens{
   int len;
   struct domain *p;
   struct nucnode *s;
   char *se;
 };


/* inpar contains all parameters received from par.dat 

a,b       : parameters in afine gap penalty
nuseq     : number of sequences in input
seqtype   : 0: DNA 1:protein 2:mixed (genomic)
udanc     : ? ancestral sequences should be in output
usertrae  : will tree be given (1) or should it be reconstructed (0)
udmat,udrest
screen    : how much information should be sent to screen while running ?
*/

struct inpar{
   int alseq,a,b,nuseq,seqtype,udanc,usertrae,udmat,udrest,screen;
  };

struct globa{
  int aligtot,totno,totdist,maxlen,lave,nmax,nmaxl,tota;
};

/* param contains parameters dependent on whether the sequences is DNA or 
   protein */

struct param{
   int signnu,i1,i2,i3,i4,nd,as,sl,ts;
   float la,ad,d,d1,d2;
   char acids[MAXSIGNS];
  };

/* pres contains variables nesc. for prealignment */

struct pres{
   struct elem *hash;
   struct alinode *alipoi;
   int *posh,upno,dono,alinum;
   int puffno,(*puffpos)[2],shores[2][SHLEN];
};

struct esko{
   int used, maxused, *ladder, *dx, *dy, *row;
 };

struct back{
   int blokno,blok[200], presblok, edgeno, (*edges) [6];
   int ltot,ltotr,inpuff,enough,pathno1,pathno2,beg,slut,reached;
   struct nucnode *aux;
 };

struct outstat{
   int conssite, substi[MAXSIGNS][MAXSIGNS], indell, indelpos;
   int (*poiar) [3];
   struct integlist indels, *indelpoint;
 };

